
package umontreal.iro.lecuyer.tcode;

import org.apache.tools.ant.util.*;
import org.apache.tools.ant.*;
import org.apache.tools.ant.taskdefs.Execute;
import org.apache.tools.ant.taskdefs.LogStreamHandler;
import org.apache.tools.ant.types.Commandline;
import org.apache.tools.ant.types.FileList;
import org.apache.tools.ant.types.FileSet;
import java.util.Enumeration;
import java.util.Vector;
import java.io.IOException;
import java.io.File;

public class Texjava extends Task {
   // We want the task as Ant-friendly as possible, so
   // use built-in file sets and file lists.
   private Vector texFileSets = new Vector();
   private Vector texFileLists = new Vector();
   private File texjava;  // Location of texjava.pl
   private File masterFile = null;
   private File htmloutdir = null;
   private boolean html = false;
   private boolean htmlset = false;
   private boolean images = false;
   private boolean imagesset = false;
   private boolean savelatex = false;
   private boolean savelatexset = false;

   private boolean htmlonly = false;
   private String htmltitle;
   private boolean overviewtopackage = false;

   private File[] texjavaList;

   private Boolean getBoolProperty (String name) {
      String val = getProject().getProperty (name);
      if (val == null)
         return null;
      if (val.equalsIgnoreCase ("yes") || val.equalsIgnoreCase ("true") ||
          val.equals ("1"))
         return new Boolean (true);
      else if (val.equalsIgnoreCase ("no") || val.equalsIgnoreCase ("false") ||
               val.equals ("0"))
         return new Boolean (false);
      else
         return null;
   }

   public boolean getHtml() {
      if (!htmlset) {
         Boolean b = getBoolProperty ("texjava.html");
         if (b != null) {
            html = b.booleanValue();
            htmlset = true;
         }
      }
      return html;
   }


   public void setHtml (boolean html) {
      this.html = html;
      htmlset = true;
   }


   public boolean getImages() {
      if (!imagesset) {
         Boolean b = getBoolProperty ("texjava.images");
         if (b != null) {
            images = b.booleanValue();
            imagesset = true;
         }
      }
      return images;
   }


   public void setImages (boolean images) {
      this.images = images;
      imagesset = true;
   }


   public boolean getSavelatex() {
      if (!savelatexset) {
         Boolean b = getBoolProperty ("texjava.savelatex");
         if (b != null) {
            html = b.booleanValue();
            savelatexset = true;
         }
      }
      return savelatex;
   }


   public void setSavelatex (boolean savelatex) {
      this.savelatex = savelatex;
      savelatexset = true;
   }


   public boolean getHtmlonly() {
      return htmlonly;
   }


   public void setHtmlonly (boolean htmlonly) {
      this.htmlonly = htmlonly;
   }


   public boolean getOverviewtopackage() {
      return overviewtopackage;
   }


   public void setOverviewtopackage (boolean pack) {
      overviewtopackage = pack;
   }


   public File getMaster() {
      return masterFile;
   }


   public void setMaster (File masterFile) {
      this.masterFile = masterFile;
   }


   public File getHtmloutdir() {
      if (htmloutdir == null)
         htmloutdir = getProject().resolveFile (getProject().getProperty ("texjava.htmloutdir"));
      return htmloutdir;
   }


   public void setHtmloutdir (File htmloutdir) {
      this.htmloutdir = htmloutdir;
   }


   public String getHtmltitle() {
      if (htmltitle == null)
         htmltitle = "";
      return htmltitle;
   }


   public void setHtmltitle (String title) {
      htmltitle = title;
   }


   public File getTexjava() {
      if (texjava == null) {
         String val = getProject().getProperty ("texjava.texjava");
         if (val != null)
            texjava = getProject().resolveFile (val);
      }
      if (texjava == null)
         texjava = new File (getProject().getBaseDir() + "/tcode", "texjava.pl");
      return texjava;
   }


   public void setTexjava (File texjava) {
      this.texjava = texjava;
   }


   public void addTexfileset (FileSet fs) {
      texFileSets.addElement (fs);
   }


   public void addTexfilelist (FileList fl) {
      texFileLists.addElement (fl);
   }


   public void execute() throws BuildException {
      resetFileLists();

      // Get all the files from the sets and lists and
      // fill in texjavaList with the files that need
      // to be processed.
      Enumeration enumTexFileSets = texFileSets.elements();
      while (enumTexFileSets.hasMoreElements()) {
         FileSet fs = (FileSet)enumTexFileSets.nextElement();
         DirectoryScanner ds = fs.getDirectoryScanner (getProject());
         String[] files = ds.getIncludedFiles();
         scanDir (fs.getDir (getProject()), files);
      }
      Enumeration enumTexFileLists = texFileLists.elements();
      while (enumTexFileLists.hasMoreElements()) {
         FileList fl = (FileList)enumTexFileLists.nextElement();
         String[] files = fl.getFiles (getProject());
         scanDir (fl.getDir (getProject()), files);
      }

      // Since texjava.pl can process one file at a time,
      // we need to run it for every file.
      GlobPatternMapper m = new GlobPatternMapper();
      m.setFrom ("*.tex");
      m.setTo (getHtmlonly() ? "*.html" : "*.java");
      if (texjavaList.length > 0 && !overviewtopackage)
         log ((htmlonly ? "Creating HTML contents from " :
               "Extracting Java code from ") + texjavaList.length +
              " TeX file" + (texjavaList.length == 1 ? "" : "s"));
      for (int i = 0; i < texjavaList.length; i++) {
         if (texjavaList[i] == null)
            continue;
         try {
            log ("Processing " + texjavaList[i].getAbsolutePath());
            Commandline cmd = new Commandline();
            // This could be replaced by an absolute
            // path to Perl interpreter which would have to
            // be determined or specified at runtime.
            cmd.setExecutable ("perl");
            // The first argument must be the name of the script
            cmd.createArgument().setFile (getTexjava());
            if (masterFile != null) {
               cmd.createArgument().setValue ("-master");
               cmd.createArgument().setFile (masterFile);
            }
            cmd.createArgument().setValue (getHtml() ? "-html" : "-nohtml");
            cmd.createArgument().setValue (getImages() ? "-images" : "-noimages");
            cmd.createArgument().setValue (getSavelatex() ? "-savelatex" : "-nosavelatex");
            if (getHtmloutdir() != null) {
               cmd.createArgument().setValue ("-htmloutdir");
               cmd.createArgument().setFile (htmloutdir);
            }
            if (getHtmlonly()) {
               cmd.createArgument().setValue ("-htmlonly");
               if (!getHtmltitle().equals ("")) {
                  cmd.createArgument().setValue ("-htmltitle");
                  cmd.createArgument().setValue (htmltitle);
               }
            }
            // Pass the TeX file
            cmd.createArgument().setFile (texjavaList[i]);
            // The .tex file must be converted to .java file
            String javaName = getOverviewtopackage()
               && texjavaList[i].getName().equals ("overview.tex")
               ? "package.html" : 
               m.mapFileName (texjavaList[i].getName())[0];
            cmd.createArgument().setFile (new File (texjavaList[i].getParentFile(), javaName));

            // We prefer to use Execute rather than JDK Runtime
            // because Execute is designed to face
            // portability issues and can redirect outputs
            // into Ant.
            Execute exe = new Execute
               (new LogStreamHandler
                (this, Project.MSG_VERBOSE, Project.MSG_WARN));
            exe.setAntRun (getProject());
            exe.setWorkingDirectory (getProject().getBaseDir());
            exe.setCommandline (cmd.getCommandline());
            exe.execute();
            if (exe.getExitValue() != 0)
               throw new BuildException
                  ("Error running texjava", getLocation());
         }
         catch (IOException e) {
            throw new BuildException
               ("Error running texjava", e, getLocation());
         }
      }
   }


   private void resetFileLists() {
      texjavaList = new File[0];
   }

   /*
    * Given a source directory and a list of .tex files,
    * scans the corresponding .java files in srcDir
    * to locate any outdated .java file that need to be
    * regenerated from its .tex file.
    */
   private void scanDir (File srcDir, String[] files) {
      GlobPatternMapper m = new GlobPatternMapper();
      m.setFrom ("*.tex");
      m.setTo (htmlonly ? "*.html" : "*.java");
      SourceFileScanner sfs = new SourceFileScanner (this);
      File[] newFiles = sfs.restrictAsFiles (files, srcDir, srcDir, m);
      if (overviewtopackage) {
         for (int i = 0; i < newFiles.length; i++) {
            if (newFiles[i].getName().equals ("overview.tex")) {
               long lastModifTex = newFiles[i].lastModified();
               File packagehtml = new File (newFiles[i].getParent(),
                                            "package.html");
               long lastModifHtml = packagehtml.lastModified();
               if (lastModifHtml >= lastModifTex)
                  newFiles[i] = null;
            }
         }
      }

      if (newFiles.length > 0) {
         File[] newTexjavaList = new File[texjavaList.length + newFiles.length];
         System.arraycopy (texjavaList, 0, newTexjavaList, 0, texjavaList.length);
         System.arraycopy (newFiles, 0, newTexjavaList, texjavaList.length, newFiles.length);
         texjavaList = newTexjavaList;
      }
   }
}
